
from img2sh.renderer import Renderer
from img2sh.pallette import XTERM_PALLETTE


def test_jpeg():
    renderer = Renderer("examples/chrome.jpeg", XTERM_PALLETTE, wsize=80)
    assert renderer.error is None
    result_img = renderer.render()
    f_true = open("examples/chrome.jpeg.bin", "r+")
    f_false = open("examples/chrome.png.bin", "r+")
    assert result_img is not None
    assert len(result_img) == 48168
    assert f_true.read() == result_img
    assert f_false.read() != result_img


def test_png():
    renderer = Renderer("examples/chrome.png", XTERM_PALLETTE, wsize=80)
    assert renderer.error is None
    result_img = renderer.render()
    f_true = open("examples/chrome.png.bin", "r+")
    f_false = open("examples/chrome.jpeg.bin", "r+")
    assert result_img is not None
    assert len(result_img) == 39780
    assert f_true.read() == result_img
    assert f_false.read() != result_img



"""rectangle module extended for navigation functions"""


class Rect():
    """Rectangle helper class
    """

    def __init__(self, cor_x_1, cor_y_1, cor_x_2, cor_y_2):
        self.x_1 = cor_x_1
        self.y_1 = cor_y_1
        self.x_2 = cor_x_2
        self.y_2 = cor_y_2
        self.width = self.x_2 - self.x_1
        self.height = self.y_2 - self.y_1

    def __repr__(self):
        return "{} {} - {} {}".format(
            self.x_1,
            self.y_1,
            self.x_2,
            self.y_2
        )

    def zoom_rect(self):
        """zoom_rect

        Returns:
            Rect: mid zoomed rect
        """
        return Rect(
            self.x_1 + (0.25 * self.width),
            self.y_1 + (0.25 * self.height),
            self.x_1 + (0.75 * self.width),
            self.y_1 + (0.75 * self.height)
        )

    def left_rect(self):
        """left_rect

        Returns:
            Rect: left rectangle
        """
        return Rect(
            self.x_1 - (0.25 * self.width),
            self.y_1,
            self.x_2 - (0.25 * self.width),
            self.y_2
        )

    def right_rect(self):
        """right_rect

        Returns:
            Rect: right rectangle
        """
        return Rect(
            self.x_1 + (0.25 * self.width),
            self.y_1,
            self.x_2 + (0.25 * self.width),
            self.y_2
        )

    def up_rect(self):
        """up_rect

        Returns:
            Rect: up rectangle
        """
        return Rect(
            self.x_1,
            self.y_1 - (0.25 * self.height),
            self.x_2,
            self.y_2 - (0.25 * self.height)
        )

    def down_rect(self):
        """down_rect

        Returns:
            Rect: down rectangle
        """
        return Rect(
            self.x_1,
            self.y_1 + (0.25 * self.height),
            self.x_2,
            self.y_2 + (0.25 * self.height)
        )



"""renderer module"""
import os
from six.moves import input
from colored import stylize, bg
from PIL import Image
import numpy as np

from .rect import Rect


def find_nearest_color(color, pallette):
    if len(color) == 3:
        (colorr, colorg, colorb) = color
    elif len(color) == 4:
        (colorr, colorg, colorb, alpha) = color
        if alpha == 0:
            return None  # pallette.index((255,255,255))
    colors = (colorr, colorg, colorb)
    diff = pallette - colors
    diff_sum = np.sum(np.square(diff), axis=1)
    return np.argmin(diff_sum)


def get_terminal_size():
    rows, columns = os.popen('stty size', 'r').read().split()
    return int(rows), int(columns)


class Renderer():
    """Renderer class which render images
    according to the specified color pallette
    """
    FONT_RATIO = 2.0
    ERROR_IMAGE_READ = 0
    ERROR_RENDER = 1

    def __init__(self, file_name, color_pallette, wsize=None):
        self.error = None
        if wsize is None:
            _, wsize = get_terminal_size()
        try:
            self.img = Image.open(file_name)
            self.wsize = wsize
            self.color_pallette = color_pallette
            self.render_count = 0
            self.img_x, self.img_y = self.img.size
            # no crop at the beginning
            self.crop = Rect(0, 0, self.img_x, self.img_y)
        except:
            print("ERROR: Image read")
            self.error = self.ERROR_IMAGE_READ

    def render(self, crop=None):
        """render the image

        Args:
            crop (Rect, optional): crop image according to the given rect. Defaults to None.

        Returns:
            str: the result string
        """
        self.render_count += 1
        img_r = self.img
        if crop is not None:
            self.crop = crop
            img_r = img_r.crop((crop.x1, crop.y1, crop.x2, crop.y2))
        img_rx, img_ry = img_r.size
        wpercent = (self.wsize/float(img_rx))
        hsize = int((float(img_ry)*float(wpercent))/self.FONT_RATIO)
        img_r = img_r.resize((self.wsize, hsize), Image.ANTIALIAS)
        self._convert_string(img_r)
        return self.image_string

    def show(self, interactive=False):
        """clear the screen and renders the image

        Args:
            interactive (bool, optional): is interactive. Defaults to False.
        """
        os.system("clear")
        print(self.image_string)
        print("[Crop]: ", self.crop)
        if interactive:
            self._interactive()

    def _convert_string(self, img_r):
        img_x, img_y = img_r.size
        image_string = "\n"
        for j in range(img_y):
            for i in range(img_x):
                color = find_nearest_color(
                    img_r.getpixel((i, j)), self.color_pallette)
                if color is None:
                    image_string += " "
                else:
                    color_value = bg(color)
                    image_string += stylize(" ", color_value)
            image_string += "\n"
        self.image_string = image_string

    def _interactive(self):
        cmd = input(
            "q: quit z: zoom+ x: zoom- c: reset \narrow keys for navigation \ncmd: ")
        if cmd == "q":
            return
        if cmd == "z":
            self.render(
                crop=self.crop.zoom_rect()
            )
        elif cmd == "x":
            self.render(
                crop=Rect(0, 0, self.img_x, self.img_y)
            )
        elif cmd == "c":
            self.render(
                crop=Rect(0, 0, self.img_x, self.img_y)
            )
        elif cmd == '\x1b[A':  # up
            self.render(crop=self.crop.up_rect())
        elif cmd == '\x1b[B':  # down
            self.render(crop=self.crop.down_rect())
        elif cmd == '\x1b[C':  # right
            self.render(crop=self.crop.right_rect())
        elif cmd == '\x1b[D':  # left
            self.render(crop=self.crop.left_rect())
        else:
            print("Unkown", cmd)
        self.show(interactive=True)
        # print(self.render_count)



#!/usr/bin/env python
# coding=utf-8

from setuptools import setup, find_packages

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setup(
    name='img2sh',
    version=__import__('img2sh').__version__,
    install_requires=requirements,
    description=(
        'Show images directly on terminal using Xterm colors.'
    ),
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author="Mehmet Ozan Unal",
    author_email='mehmetozanunal@gmail.com',
    maintainer="Mehmet Ozan Unal",
    maintainer_email='mehmetozanunal@gmail.com',
    license="MIT",
    packages=find_packages(),
    platforms=["all"],
    url='https://github.com/mozanunal/img2sh',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development :: Libraries'
    ],
    entry_points={
        'console_scripts': ['img2sh=img2sh.cli:main'],
    },
)



"""cli module for img2sh"""
from __future__ import print_function
from .renderer import Renderer
from .pallette import XTERM_PALLETTE


def is_link(test_string):
    """check if the string is a web link

    Args:
        test_string (str): input string

    Returns:
        bool: if string is an http link
    """
    return test_string.startswith('http')


def main():
    """main cli function
    """
    import argparse

    parser = argparse.ArgumentParser(
        description='Show images directly on terminal.')
    parser.add_argument(
        "Image", help="the directory of the image which will be opened")
    parser.add_argument(
        "-w", "--width",
        help="image width on the terminal",
        type=int
    )
    parser.add_argument(
        "-i", "--interactive",
        default=False, action='store_true',
        help="open image in interactive mode",
    )

    args = parser.parse_args()

    renderer = Renderer(args.Image, XTERM_PALLETTE, wsize=args.width)
    if renderer.error is None:
        renderer.render()
        renderer.show(interactive=args.interactive)


if __name__ == "__main__":
    main()
